'''
Created on 05 Oct 2011

@author: Simon Wilshin, swilshin@rvc.ac.uk

This class will compute velocities from data containers.
When instatiated will report the keys of the velocities 
it can calculate under the same name as the data being 
differentiated / integrated appended suitably.
Presently only recognises data from the motionCaptureData
object
'''

from baseAnalysis import BaseAnalysis
from mDInterpret import getID
from utils import numericallyDifferentiate

class SampleRateMismatch(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class Velocity(BaseAnalysis):
    defaultTargets = ["*tMotionCaptureData"]
    defaultIDExt = "vel"
    def __init__(self, workingDirectory, analysisDirectory, metaData, parent):
        BaseAnalysis.__init__(self, workingDirectory, analysisDirectory, metaData, parent)
        self.sources = [self.parent.resolve(t) for t in self.targets]
        self.sampleRate = self.sources[0].sampleRate
        if not all([s.sampleRate==self.sources[0].sampleRate for s in self.sources]):
            raise(SampleRateMismatch("Sample rates do not match in velocity object!"))
        self.strideFrequencyEstimate = self.metaData["strideFrequencyEstimate"]
        self.maxFrequencyOfInterestMultiplier = self.metaData["maxFrequencyOfInterestMultiplier"]
        if self.metaData.has_key("filterOrder"):
            self.filterOrder = self.metaData["filterOrder"]
        else:
            self.filterOrder = 5

    def _getVelocity(self,k):
        '''
        Calcualte the velocities  of all markers.
        '''
        # Note that you must assign directly to the data dictonary here to avoid infinite recursion
        for s in self.sources:
            if s.has_key(k):
              self.data["_".join([k,"velocity"])]=numericallyDifferentiate(s[k],self.sampleRate,self.strideFrequencyEstimate,self.maxFrequencyOfInterestMultiplier,self.filterOrder)

    def has_key(self,k):
        '''
        Check if key k refers to a velocity we can calculate, just checks if it is in keys().
        @param k: Which key to compute the velocity of
        @type k: string 
        '''
        return(k in self.keys())
        
    def keys(self):
        return(["_".join([k,"velocity"]) for s in self.sources for k in s.keys() ])

    def __getitem__(self,k):
        '''
        Returns an element of the dictionary self.data based on the key k 
        which is mangled (projected from a list of acceptable key names on 
        to the space of actual key names in data) first so the user has a 
        variety of sensible ways to get at the data. Will throw an exception 
        if k does not correspond to anything after mangling.
        Because certain quantities are computationally expensive to determine
        this method now checks to see if these have been calculated when 
        accessed. If they have not it calls the method to compute them. This 
        avoids the need to determine the at start up.
        Returns: The data associated with k.
        '''
        # Mangle the key first
        nk=self.mangleKey(k)
        if (type(nk)==type(str()) or type(nk)==type(u"")):
            if nk[-9:]=="_velocity":
                if not self.data.has_key(nk):
                    self._getVelocity(nk[:-9])
        return(self.data[k])
