'''
Created on 05 Oct 2011

@author: Simon Wilshin, swilshin@rvc.ac.uk

Two classes, one which builds a phaser based on some data, another 
which links to that phasers and calculates phases for some data set.
'''

import numpy as np
from numpy import nanmean

from baseAnalysis import BaseAnalysis
from mDInterpret import getID
from mDInterpret import loadMetaData
from utils.protophases import VXKalmanPhase

class VelPhase(BaseAnalysis):
    defaultTargets = ["*tMarkerSet","*tVelocity"]
    defaultIDExt = "rph"
    def __init__(self, workingDirectory, analysisDirectory, metaData, parent):
        BaseAnalysis.__init__(self, workingDirectory, analysisDirectory, metaData, parent)
        self.markers = self.parent.resolve(self.targets[0])
        self.vel = self.parent.resolve(self.targets[1])
        self.sampleRate = self.vel.sampleRate
        loadMetaData("yscale",self,self.metaData,0.1)
        self.phi = VXKalmanPhase(self.sampleRate, self.yscale)
        
    def _getPhase(self,k):
        '''
        Calcualte the velocities  of all markers.
        '''
        # Get the toe
        if k == "Body":
            t = self.vel["CoM_velocity"][2]
            t -= nanmean(t)
            t = np.reshape(t,(1,t.size))
            self.data[k+"_phase"] = np.angle(self.phi(t))
        else:
            t = self.vel[self.markers[k+"Markers"][0]+"_velocity"][2]
            t -= nanmean(t)
            t = np.reshape(t,(1,t.size))
            self.data[k+"_phase"] = np.angle(self.phi(t))

    def has_key(self,k):
        return(k in self.keys())

    def keys(self):
        return(["Body_phase"]+["_".join([k,"phase"]) for k in self.markers["Appendages"]])

    def __getitem__(self,k):
        '''
        Returns an element of the dictionary self.data based on the key k 
        which is mangled (projected from a list of acceptable key names on 
        to the space of actual key names in data) first so the user has a 
        variety of sensible ways to get at the data. Will throw an exception 
        if k does not correspond to anything after mangling.
        Because certain quantities are computationally expensive to determine
        this method now checks to see if these have been calculated when 
        accessed. If they have not it calls the method to compute them. This 
        avoids the need to determine the at start up.
        Returns: The data associated with k.
        '''
        # Mangle the key first
        nk=self.mangleKey(k)
        if (type(nk)==type(str()) or type(nk)==type(u"")):
            if nk[-6:]=="_phase":
                if not self.data.has_key(nk):
                    self._getPhase(nk[:-6])
        return(self.data[k])  

