'''
Created on 18 Jan 2011

@author: Simon Wilshin, swilshin@rvc.ac.uk

Configuration file that ensures when the dataContainer package is imported 
everything of use is pulled in.

Further details created previously on 19 Nov 2010

@author: Simon Wilshin, swilshin@rvc.ac.uk

This module handles biomechanical data objects. At present this includes force
plate and optical motion capture data, gathered at the Structure and Motion
Laboratory, Royal Veterinary College, London.

Un-named bugs / feature requests waiting to be fixed / implemented:
These are the lowest priority portions of the code, they are general ideas or low 
priority bugs. They are marked by who originated them
TYPE: Desciption - ORIGINATOR - OPTIONAL TAG
FEATURE REQUEST: Add code which creates missing directories on file writes - ANNA
FEATURE REQUEST: Default names for qualysisNames from filename - ANNA
FEATURE REQUEST: Better trough and peak detection by determining when we fall 
                 below a threshold then determining when we rise back above it - SIMON
FEATURE REQUEST: Instead of just doing trots we should mark walks and other gaits 
                 (which should depend on the polypedal class of the animal) 
                 on the phase plots - SIMON - FR0001
NEEDED BUG FIX: Peak detection uses a local method, should be non-local - SIMON
FACTORISATION: Peak detection for plates and in general has duplicate code - SIMON
FACTORISATION: MotionCaptureData.makePlots has overlapping code with 
               MotionCaptureData._doPlots - SIMON 
Named bugs:
These are high priority bugs or features that are required post haste. The have an 
originator and someone should be assigned responsibility for them ASAP.
NAME:    Description - ORIGINATOR - RESPONSIBILITY
BUG0001: Unspecified format for force plate data. The format of the raw force 
         plate data files should be specified somewhere. - SIMON - UNASSIGNED
BUG0002: Try dPhi by dt for local stride frequency. The footfall detection 
         algorithm works exceptionally poorly on 
         treadmill data because the feet are not stationary in the reference 
         frame of the world but in the treadmill frame. Since we are looking 
         at gait transitions we should find some local estimate of forward 
         velocity and subtract this - SIMON - UNASSIGNED
BUG0003: See dPhi dt in BUG0002, On data with gait transitions there is no 
         single stride frequency, but a 
         single one is extracted and the algorithm we use assumes only one such 
         stride frequency (since it takes the median power spectrum of the 
         longest non-NaN sequence of data). What we should be doing if the 
         stride frequency matters (and it does for any application requiring 
         harsh filtering) is calculating a running estimate of the stride 
         frequency. As a first attempted fix I've rewritten the code to look 
         when the acceleration is stationary for treadmill data. Shockingly 
         this worked pretty well for the sample data I have, Spence should check 
         if it work for other horse data - SIMON - SPENCE
BUG0004: The outlier metric method of finding spurious footfalls performs badly 
         when there is a gait transtion - SIMON - UNASSIGNED
BUG0005: The phase extraction algorithm is in two sections (overground and 
         treadmill), I'm (SIMON) not sure why. This bug should now encorporate 
         dealing with drift of the animal on the treadmill by lowpass filtering 
         and subtracting the centroid estimate of the CoM.- SIMON - SIMON
BUG0006: Currently we use a frame of reference for motion in which x,
         y and z are just ripped right out of the motion capture data. We 
         should really establish some kind of body co-ordinate system, this 
         may have to be a running co-ordinate system - SIMON - SIMON
'''

from baseData import *
from forcePlateData import *
from motionCaptureData import *
from nerveBlockData import *
from substrateData import *
from bandageConditionData import *
from gaitTransitionData import *
from stepPerturbationData import *
from textData import *
from huntingdonsStatus import *
from weightInterface import *
from imuData import *
from gPIntMoCapData import *
from splineIntMoCapData import *
from nGpIntMoCapData import *
#from xrlData import *

def runTests():
    '''
    Runs some tests using the sample data
    '''
    print "\nProcessing sample bandage data"
    sampleBandageConditionData=getSampleBandageConditionData()
    print sampleBandageConditionData.bandagedLegs    
    
    print "\nProcessing sample force plate data"
    sampleForcePlateData=getSampleForcePlateObject()
    sampleForcePlateData.makePlots()
    print "\nProcessing sample force plate weight object"
    sampleForcePlateWeight=getForcePlateWeightObject()
    print "Weight of sample dog is ", sampleForcePlateWeight.weight, "N that is ", sampleForcePlateWeight.mass, "kg."
    sampleForcePlateWeight.makePlots()
    
    print "\nProcessing sample motion capture data (dog, mat)"
    sampleQualisysData=getSampleMotionCaptureObject()
    sampleQualisysData.makePlots()
    print "\nProcessing sample motion capture data (dog, tsv)"
    sampleQualisysDataTSV=getSampleTSVMotionCaptureObject()
    sampleQualisysDataTSV.makePlots() 
    print "\nProcessing sample motion capture data (horse, mat)"
    sampleHorseTSVData = getSampleHorseTSVMotionCaptureObject()
    sampleHorseTSVData.makePlots()
    print "\nProcessing sample motion capture data with gait transition (horse, first sample)"
    sampleTransitionData = getSampleHorseGaitTransition(0)
    sampleTransitionData.makePlots()
    print "\nProcessing sample motion capture data with gait transition (horse, second sample)"    
    sampleTransitionData = getSampleHorseGaitTransition(1)
    sampleTransitionData.makePlots()
    print "\nProcessing sample motion capture data with gait transition (dog)"    
    sampleTransitionData = getSampleDogGaitTransition()
    sampleTransitionData.makePlots()
    print "\nProcessing sample motion capture data (fish)"    
    sampleFishData = getSampleFishMotionCaptureObject()
    sampleFishData.makePlots()

    print "\nProcessing sample nerve block data"
    sampleNerveBlockData = getSampleNerveBlockData()
    print "Blocked legs: ", sampleNerveBlockData.blockedLegs
    print "Block type: ", sampleNerveBlockData.blockType

    print "Processing sample gait transition data"
    sampleGaitTransitionData=getSampleGaitTransitionData()
    print "Gait Transition: ", sampleGaitTransitionData.transitionType, "."    

    print "\nProcessing sample compliant substrate data"
    sampleSingleCompliantSubstrateData = getSampleSingleCompliantSubstrateData()
    print "Substrate Name: ", sampleSingleCompliantSubstrateData.name
    print "Substrate k: ", sampleSingleCompliantSubstrateData.k     
       
    print "\nProcessing sample step perturbation data"
    sampleStepPerturbationData = getSampleStepPerturbationData()
    print "Step Height: ", sampleStepPerturbationData.height, "mm."
    
    print "Processing sample nerve block data"
    sampleHuntingdonsStatusData=getSampleHuntingdonsStatusData()
    print "Huntingdon's Status: ", sampleHuntingdonsStatusData.huntingdonsStatus
    
    print "Processing sample marker sets"
    testSampleMarkerSet()
    testHorseLabels()
    testDogLabels()

    print "Processing sample fish marker set"
    testFishMarkerSet()

if __name__=='__main__':
    runTests()