import scipy.cluster.hierarchy as hrchy
import numpy as np

from mDInterpret import loadMetaData
from motionCaptureData import MotionCaptureData
from utils import GPInterpolater, ensure_dir, fillNans
import pylab as pl
from os.path import join as osjoin

class GPIntMoCapData(MotionCaptureData):
    '''
    Wraps the MotionCaptureData in methods that use a gaussian process
    to fill in blanks.
    '''
    def __init__(self, workingDirectory, analysisDirectory, metaData, parent):
        MotionCaptureData.__init__(self, workingDirectory, analysisDirectory, metaData, parent)
        loadMetaData("Nmeas",self,self.metaData,200)
        loadMetaData("logHyper",self,self.metaData,None)
        loadMetaData("localityWeight",self,self.metaData,100.0)
        
    def keys(self):
        return([k for k in self.data.keys() if k[-6:]!="_gpint"])
        
    def has_key(self,k):
        return(k in self.keys())
    
    def interpolate(self,k):
        # For the signal k, find the regions that are NaNs.
        sigs = self.data[k]
        nSigs = []
        covFunc = ['kernels.covSum', ['kernels.covSEard','kernels.covNoise']]
        '''['kernels.covSum',
            ['kernels.covSEiso1only','kernels.covSEiso2plus','kernels.covNoise']
        ]'''
        for s in sigs:
            newS = np.array(s)
            if sum(np.isnan(s))>1:
                # Get the missing times
                missTau = self.time[np.isnan(s)]
                missIdx = np.array(range(self.time.size))[np.isnan(s)]
                t0 = self.time[~np.isnan(s)]
                X = s[~np.isnan(s)]
                # Find distance matrix and cluster
                E = missTau.repeat(missTau.size)
                E = E.reshape(missTau.size,missTau.size)
                Delta = np.abs(E-E.transpose())
                l = hrchy.linkage(Delta)
                cl = hrchy.fcluster(l,1.0)
                # For every cluster, fill with a gaussian process
                for idxs in set(cl):
                    mt = missTau[idxs==cl]
                    w = -(
                          np.repeat(t0,mt.shape[0]).reshape((t0.shape[0],mt.shape[0]))
                          - np.repeat(mt,t0.shape[0]).reshape((mt.shape[0],t0.shape[0])).transpose()
                         )**2
                    w = np.sum(w,1)
                    w -= np.max(w)
                    w = w.astype(np.float32)
                    w /= float((t0[-1]-t0[0])**2.0)
                    w = np.exp(self.localityWeight*w)
                    w *= np.random.rand(w.size)
                    w = sorted(zip(w,range(w.size)),key = lambda x: x[0])
                    tt = t0[np.array(zip(*w[-self.Nmeas:])[1])]
                    gX = X[np.array(zip(*w[-self.Nmeas:])[1])]
                    if self.logHyper==None:
                        gp = GPInterpolater(tt.reshape((tt.size,1)),gX.reshape((gX.size,1)),covFunc,np.array([-1.0,-1.0,-1.0]),train=True)
                    else:
                        gp = GPInterpolater(tt.reshape((tt.size,1)),gX.reshape((gX.size,1)),covFunc,self.logHyper,train=False)
                    mX = gp.predict(mt.reshape((mt.size,1)))[0]
                    newS[missIdx[idxs==cl]] = mX
            elif sum(np.isnan(s))==1:
                newS=fillNans(s)
            nSigs.append(newS)
        self.data[k+"_gpint"] = np.vstack(nSigs)
    
    def __getitem__(self,k):
        if not self.data.has_key(k+"_gpint"):
            if self.metaData["verbosity"]>0:
                print "  GP interpolating %s" % k
            self.interpolate(k)
        return(self.data[k+"_gpint"])
        
