'''
Created on 18 May 2011

@author: swilshin
'''

import csv
from csv import reader
import pylab
import mpl_toolkits.mplot3d.axes3d as p3
from os.path import join as osjoin
from yaml import load as yamlload
import numpy as np
from numpy import nanstd
from numpy import nanmean
from scipy.interpolate import UnivariateSpline

from phaser import Phaser

import conventions as conv
from utils import angularCentralTendancy
# maybe utils vs sipylibs version of FiltFiltFilter made difference?
#from utils import FiltFiltFilter
from utils import nanInterpolation
from utils import wrappedInterpolate
from baseData import BaseData
from temporalSignal import TemporalSignal
from sipylibs import FiltFiltFilter

filtFreq = 1.0

def zscore(x):
    return((x-nanmean(x))/nanstd(x))

def roughTerrainPSec(x):
    filt = FiltFiltFilter(100,filtFreq,3) # Hardcoded, BAD!
    sig = filt.filter(x[0])
    return(sig)

class IMUData(BaseData):
    '''
    Stores data from IMU, can apply phaser algorithm to it.
    Behaves in a very similar manner to the motionCaptureData class.
    '''
    defaultIDExt = "imu"
    def singlePlotPhases(self,limbs,subRegion=False):
        colAcc={'FL':"r-",'BR':"r-."}
        colGyro={'FL':"b-",'BR':"b-."}
        colPh = {'FL':"g-",'BR':"g-."}
        if subRegion==True:
            fIdx = self.firstStrideIdx 
            lIdx = self.lastStrideIdx
        else:
            fIdx = 0
            lIdx = -1
        pylab.figure()
        numberOfFigures=3
        pylab.gcf().set_size_inches( conv.pylabImageSizeX, conv.pylabImageSizeY*numberOfFigures)
        pylab.subplot(conv.pylabColumnMultiplier*numberOfFigures+conv.pylabRowMultiplier+2)
        pylab.title("Absolute phase")
        pylab.xlabel("time (s)")
        pylab.ylabel("absolute phase (rad)")
        for limb in limbs:
            pylab.plot(self.masterTime[fIdx:lIdx]-np.min(self.masterTime[fIdx:lIdx]),self[limb+"Phase"][fIdx:lIdx],colPh[limb])
        pylab.ylim(0,2.0*np.pi)
        pylab.subplot(conv.pylabColumnMultiplier*numberOfFigures+conv.pylabRowMultiplier+3)
        pylab.title("Relative phase")
        pylab.xlabel("time (s)")
        pylab.ylabel("relative phase (rad)")      
        for l1,l2 in zip(limbs[:-1],limbs[1:]):
            pylab.plot(self.masterTime[fIdx:lIdx]-np.min(self.masterTime[fIdx:lIdx]),self[l1+l2+"RelativePhase"][fIdx:lIdx],"k-")
        pylab.ylim(0,2.0*np.pi)  
        pylab.subplot(conv.pylabColumnMultiplier*numberOfFigures+conv.pylabRowMultiplier+1)
        pylab.title("Z-scored filtered accelerometer and gyro signals")
        pylab.xlabel("time (s)")
        pylab.ylabel("gyro / accelerometer signal")        
        for l in limbs:
            pylab.plot(self.masterTime[fIdx:lIdx]-np.min(self.masterTime[fIdx:lIdx]),self.filteredSignals[l][0][fIdx:lIdx],colGyro[l])
            pylab.plot(self.masterTime[fIdx:lIdx]-np.min(self.masterTime[fIdx:lIdx]),self.filteredSignals[l][1][fIdx:lIdx],colAcc[l])
        pylab.savefig(osjoin(self.analysisDirectory, "accGyroTrace.png"))
    
    def plotPhases(self, subRegion=False):
        '''
        Make a 3D plot of the phases we obtain on a stride by stride basis for this trial
        '''
        if subRegion==True:
            fIdx = self.firstStrideIdx 
            lIdx = self.lastStrideIdx
        else:
            fIdx = 0
            lIdx = -1
        print "Plotting phases"
        pylab.figure()
        numberOfFigures=4
        pylab.gcf().set_size_inches( conv.pylabImageSizeX, conv.pylabImageSizeY*numberOfFigures)
        pylab.subplot(conv.pylabColumnMultiplier*numberOfFigures+conv.pylabRowMultiplier+3)
        pylab.title("Absolute Phases")
        pylab.xlabel("time (s)")
        pylab.ylabel("absolute phase (rad)")
        pylab.ylim(0,2.0*np.pi)
        for limb in self.limbs:
            pylab.plot(self.masterTime[fIdx:lIdx],self[limb+"Phase"][fIdx:lIdx])
        pylab.subplot(conv.pylabColumnMultiplier*numberOfFigures+conv.pylabRowMultiplier+1)
        pylab.title("Relative Phases")
        pylab.xlabel("time (s)")
        pylab.ylabel("relative phase (rad)")
        pylab.ylim(0,2.0*np.pi)
        for l1,l2 in zip(self.limbs[:-1],self.limbs[1:]):
            pylab.plot(self.masterTime[fIdx:lIdx],self[l1+l2+"RelativePhase"][fIdx:lIdx])
        pylab.subplot(conv.pylabColumnMultiplier*numberOfFigures+conv.pylabRowMultiplier+2)
        pylab.title("Filtered Signals")
        pylab.xlabel("time (s)")
        for l in self.limbs:
            pylab.plot(self.masterTime[fIdx:lIdx],self.filteredSignals[l][0][fIdx:lIdx])
            pylab.plot(self.masterTime[fIdx:lIdx],self.filteredSignals[l][1][fIdx:lIdx])
        pylab.subplot(conv.pylabColumnMultiplier*numberOfFigures+conv.pylabRowMultiplier+4)
        pylab.title("Raw Signals")
        pylab.xlabel("time (s)")
        for l in self.limbs:
            pylab.plot(self.masterTime[fIdx:lIdx],self.unfilteredSignals[l][0][fIdx:lIdx])
            pylab.plot(self.masterTime[fIdx:lIdx],self.unfilteredSignals[l][1][fIdx:lIdx])    

    
    def plot3DPhaseScatterPlot(self):
        x=self["relativeStridePhases"].transpose()[0]
        y=self["relativeStridePhases"].transpose()[1]
        z=self["relativeStridePhases"].transpose()[2]
        fig=pylab.figure()
        ax = p3.Axes3D(fig)
        ax.scatter3D(np.ravel(x),np.ravel(y),np.ravel(z),c='r')
        ax.set_xlabel('Phi1')
        ax.set_ylabel('Phi2')
        ax.set_zlabel('Phi3')
        ax.set_xlim3d([0,2.0*np.pi])
        ax.set_ylim3d([0,2.0*np.pi])
        ax.set_zlim3d([0,2.0*np.pi])
        fig.add_axes(ax)
        
    def _getPhases_legacy(self):
        '''
        Obtain the desired phases using phaser.
        AJS: Dec. 31, 2011: made redundant--new version integrates Gyro Z, and bandpasses Acc Y. Below.
        '''
        print "Evaluating phases using phaser"
        self.filteredSignals = dict()
        self.unfilteredSignals = dict()
        for limb in self.limbs:
            # Fill Nans, Interpolate onto master clock, and zscore
            sigs = list()
            osigs = list()
            filt = FiltFiltFilter(100,self.strideFrequencyEstimate,3) # HARDCODED!
            # Unfiltered signals
            for e in self[limb+self.gyro].transpose():
                osigs.append(wrappedInterpolate(self[limb+self.gyro].time,zscore(nanInterpolation(e)),self.masterTime,method="simple"))
            for e in self[limb+self.acc].transpose():
                osigs.append(wrappedInterpolate(self[limb+self.acc].time,zscore(nanInterpolation(e)),self.masterTime,method="simple"))
            self.unfilteredSignals[limb]=np.array([osigs[2],osigs[5]])
            # Filtered signal
            for e in self[limb+self.gyro].transpose():
                sigs.append(filt.filter(wrappedInterpolate(self[limb+self.gyro].time,zscore(nanInterpolation(e)),self.masterTime,method="simple")))
            for e in self[limb+self.acc].transpose():
                sigs.append(filt.filter(wrappedInterpolate(self[limb+self.acc].time,zscore(nanInterpolation(e)),self.masterTime,method="simple")))
            self.filteredSignals[limb]=np.array([sigs[2],sigs[5]])
            # Format signals for use in phaser
            filtFreq = self.strideFrequencyEstimate # Global variable very naughty
            phases = np.array(self.filteredSignals[limb])
            # Get Phaser
            self.phaser = Phaser(phases,psecfunc=roughTerrainPSec)
            # Get Phases
            self[limb+"Phase"] = self.phaser.phaserEval(phases)[0,:]%(2.0*np.pi)
        for l1,l2 in zip(self.limbs[:-1],self.limbs[1:]):
            self[l1+l2+"RelativePhase"] = (self[l1+"Phase"]-self[l2+"Phase"])%(2.0*np.pi)            

    def _getPhases(self):
        '''
        Obtain the desired phases using phaser.
        '''
        print "Evaluating phases using phaser"
        self.filteredSignals = dict()
        self.unfilteredSignals = dict()
        for limb in self.limbs:
            # Fill Nans, Interpolate onto master clock, and zscore
            sigs = list()
            osigs = list()
            cut = 0.95 # 0.95 works well, 0.1 is too low. do we need to low pass afterwards? leg pos changes direction...
            fl = FiltFiltFilter(300,cut,3,'high')
            flg = FiltFiltFilter(300,4,3); # try a low pass as 5 hz... almost for gyro
            fla = FiltFiltFilter(300,2,3); # try a low pass as 5 hz... almost for acc       
            #flai = utils.FiltFiltFilter(300,3,3);
            #filt = FiltFiltFilter(100,self.strideFrequencyEstimate,3) # HARDCODED!
            
            # Unfiltered signals
            for e in self[limb+self.gyro].transpose():
                osigs.append(wrappedInterpolate(self[limb+self.gyro].time,zscore(nanInterpolation(e)),self.masterTime,method="simple"))
            for e in self[limb+self.acc].transpose():
                osigs.append(wrappedInterpolate(self[limb+self.acc].time,zscore(nanInterpolation(e)),self.masterTime,method="simple"))
            self.unfilteredSignals[limb]=np.array([osigs[2],osigs[4]]) # use GYRO Z = 2, ACC Y = 4. ([0 1 2 3 4 5] = [0gx 1gy 2gz 3ax 4ay 5az])
            # Filtered signal
            for e in self[limb+self.gyro].transpose():
                sigs.append(zscore(np.cumsum(flg.filter(fl.filter(wrappedInterpolate(self[limb+self.gyro].time, \
                    nanInterpolation(e),self.masterTime,method="simple"))))))
                #sigs.append(filt.filter(wrappedInterpolate(self[limb+self.gyro].time,zscore(nanInterpolation(e)),self.masterTime,method="simple")))
            for e in self[limb+self.acc].transpose():
                #sigs.append(zscore(np.cumsum(fl.filter(np.cumsum(fla.filter(fl.filter(wrappedInterpolate(self[limb+self.gyro].time, \
                #    nanInterpolation(e),self.masterTime,method="simple"))))))))
                #sigs.append(zscore(flai.filter(np.cumsum(fl.filter(wrappedInterpolate(self[limb+self.acc].time, \
                #    nanInterpolation(e),self.masterTime,method="simple"))))))
                if 1: # this works well with fla=300,2,3; fl = 300,0.95,3;
                    sigs.append(zscore(fla.filter(fl.filter(wrappedInterpolate(self[limb+self.acc].time, \
                        nanInterpolation(e),self.masterTime,method="simple")))))
                #sigs.append(filt.filter(wrappedInterpolate(self[limb+self.acc].time,zscore(nanInterpolation(e)),self.masterTime,method="simple")))
            self.filteredSignals[limb]=np.array([sigs[2],sigs[4]])
            # Format signals for use in phaser
            #filtFreq = self.strideFrequencyEstimate # Global variable very naughty
            phases = np.array(self.filteredSignals[limb])
            # Get Phaser
            self.phaser = Phaser(phases)
            # Get Phases
            self[limb+"Phase"] = self.phaser.phaserEval(phases)[0,:]%(2.0*np.pi)
        for l1,l2 in zip(self.limbs[:-1],self.limbs[1:]):
            self[l1+l2+"RelativePhase"] = (self[l1+"Phase"]-self[l2+"Phase"])%(2.0*np.pi)             

    def _strideCut(self):
        # Find strides based on fore left limb phase
        speeds = wrappedInterpolate(self["GPSSpeed"].time,self["GPSSpeed"],self.masterTime,method="simple")
        self["relativeStridePhases"]=list()
        self["strideSpeeds"] = list()
        self["strideDur"]= list()
        self["strideFreq"] = list()
        self["strideLen"] = list()
        self.strideIdx = [idx for e1,e2,idx in zip((self["FLPhase"][:-1]+np.pi)%(2.0*np.pi)-np.pi,(self["FLPhase"][1:]+np.pi)%(2.0*np.pi)-np.pi,range(len(self["FLPhase"][:-1]))) if e1<0 and e2>0]
        self.firstStrideIdx = self.strideIdx[self.firstStride]
        self.lastStrideIdx = self.strideIdx[self.lastStride]
        for idx0,idx1 in zip(self.strideIdx[self.firstStride:self.lastStride-1],self.strideIdx[self.firstStride+1:self.lastStride]): # HARDCODED to ignore the first few
            self["relativeStridePhases"].append([
                                                   angularCentralTendancy(self[l1+l2+"RelativePhase"][idx0:idx1],ctf=nanmean)
                                                 for l1,l2 in zip(self.limbs[:-1],self.limbs[1:])])
            self["strideSpeeds"].append([
                                         nanmean(speeds[idx0:idx1])
                                         ])
            self["strideDur"].append([self.masterTime[idx1]-self.masterTime[idx0]])
            self["strideFreq"].append([1/(self.masterTime[idx1]-self.masterTime[idx0])])
            self["strideLen"].append([nanmean(speeds[idx0:idx1])*(self.masterTime[idx1]-self.masterTime[idx0])])                          
        self["strideSpeeds"] = np.array(self["strideSpeeds"])
        self["relativeStridePhases"] = np.array(self["relativeStridePhases"])
        self["strideDur"] = np.array(self["strideDur"])
        self["strideFreq"] = np.array(self["strideFreq"])
        self["strideLen"] = np.array(self["strideLen"])
    
    def _estDutyFactor(self):
        '''
        Inspecting. And thinking.
        GyroZ appears to have one big smooth negative hump in swing, and a flat bit with duouble humps in stance.
        The big swing appears to be flanked by swings in same direction by AccX.
        PROBLEM: Lots of trials appear to ahve sign inverted Gyro.
        '''
        f0 = 0.2 # Hz
        sfreq = 1.5 # Hz
        tol = 0.5
        lf0 = 10 # Hz
        # If using one pass, set mid level frequency,
        # otherwise use high frequency for two pass:
        if self.metaData.has_key('twopass'):
            llf0 = 3 #  low pass for two pass, stride cut only.
        else:
            llf0 = 10 # low pass for even detection entirely, one pass.
        print "Estimating duty factor"
        if self.metaData.has_key('limb'):
            limb=self.metaData['limb']
        else:
            limb='FL'
        print "Estimating duty factor"
        tau = self[limb+'Acc'].time
        # Code below wants 6xN    
        dat=np.hstack((self[limb+'Acc'],self[limb+'Gyro'])).T
        # Fix the missing values
        idx = np.arange(dat.shape[1])
        dat = np.array([UnivariateSpline(idx[np.logical_not(np.isnan(d))], \
            d[np.logical_not(np.isnan(d))],s=0)(idx) for d in dat])
        a = dat[0:3]
        g = dat[3:6]
        frq = 1.0/(tau[1]-tau[0])
        hpf = FiltFiltFilter(frq,f0,3,'highpass')
        # trick the code for now...
        lpf = FiltFiltFilter(frq,lf0,3,'lowpass')
        llpf = FiltFiltFilter(frq,llf0,3,'lowpass')

        # stride cut
        ssig=llpf(hpf(g[2]))
        if self.metaData.has_key('invertGZ'):
            ssig=-ssig
        ssiglog = np.array(ssig>0.,dtype=float)
        stcut = np.where(np.diff(ssiglog)==1)[0]
        stcuto = np.where(np.diff(ssiglog)==-1)[0]

        # Quick duty cycle calculation (an underestimate)
        # print sum(lpf(hpf(g[2]))>0.)/float(g.shape[1])
        sig=lpf(hpf(g[2]))
        print self.metaData.keys()
        if self.metaData.has_key('invertGZ'):
            print "Inverting gyro z signal."
            invertFlag="INVERTED"
            sig=-sig
        else:
            invertFlag="NOT_INVERT"
        duty = np.array(sig>0.,dtype=float)
        # Fill in gaps
        fon = np.where(np.diff(duty)==1)[0]
        foff = np.where(np.diff(duty)==-1)[0]

        if not self.metaData.has_key("twopass"):
            fonf=stcut
            fofff=stcuto
            tpflag="ONEPASS"
        else:
            tpflag="TWOPASS"
            fonf=np.empty(stcut.shape)
            for idx,onIdx in enumerate(stcut):
                fonf[idx]=fon[np.argmin(np.abs(onIdx-fon))]
            fofff=np.empty(stcuto.shape)
            for idx,offIdx in enumerate(stcuto):
                foffc=foff[foff>offIdx]
                if len(foffc) == 0:
                    print "NO FOFF AFTER THIS"
                    fofff[idx]=offIdx
                else:
                    print "FOUND FOFF AFTER THIS"
                    fofff[idx]=foffc[np.argmin(foffc-offIdx)]
                    nonc=fonf[fonf>offIdx]
                    if len(nonc) > 0:
                        if fofff[idx] > nonc[0]:
                            print "FOUND OFF WAS AFTER NEXT ON"
                            fofff[idx]=offIdx

        if False:
            figure()
            gp = lpf(hpf(g[2]))
            gp = (gp-mean(gp))/std(gp)
            plot(gp,c='r',lw=2,label="acc",alpha=0.7)
            vlines(fon,-5,5,color='k',lw=3,label="foot-on")
            vlines(foff,-5,5,color='b',lw=3,label="foot-off")
            xlim([0,2000])
            xlabel("samples")
            legend()
            ylabel("zscore gyro")
            #savefig("contactevents.png")
            title('RAW EVENTS')

        #print len(fon)
        #print len(foff)

        if False:
            idxfix = np.logical_not(np.diff(fon)<tol*(frq/sfreq))
            # works if 1 more off than on:
            fon = fon[:-1][idxfix]
            foff = foff[1:][idxfix]

        if self.parent['trialType']=='Walk':
            pylab.figure()
            sigz = (sig-np.mean(sig))/np.std(sig)
            ap = lpf(hpf(a[0]))
            ap = (ap-np.mean(ap))/np.std(ap)
            #pylab.plot(sig,c='y',lw=2,label="gyroZorig",alpha=0.7)
            pylab.plot(sigz,c='g',lw=2,label="gyroZ_z",alpha=0.7)
            pylab.plot(ap,c='r',lw=2,label="accX",alpha=0.7)
            pylab.vlines(fon,-5,5,color='k',lw=3,label="foot-on")
            pylab.vlines(foff,-5,5,color='b',lw=3,label="foot-off")
            pylab.vlines(stcut,-5,5,color='r',lw=3,label="stride")
            pylab.vlines(stcuto,-5,5,color='g',lw=3,label="strideoff")
            pylab.vlines(fonf,-5,5,color='m',lw=1,label="foot-on-final")
            pylab.vlines(fofff,-5,5,color='y',lw=1,label="foot-off-final")
            pylab.xlim([0,2000])
            pylab.xlabel("samples")
            pylab.legend()
            pylab.ylabel("zscore acc/gyro")
            pylab.title("_".join([self.parent.parent.iD,self.parent["trialType"], 
                self.parent['terrainType'],str(self.parent['roiNum']),limb,invertFlag,tpflag]))

            #savefig("contactevents.png")

        if fofff[0]<fonf[0]:
            fofff = fofff[1:]
        l0 = min([len(fonf),len(fofff)])

        fonf = fonf[:l0]
        fofff = fofff[:l0]

        df = np.array((fofff[:-1]-fonf[:-1]),dtype=float)/(fonf[1:]-fonf[:-1])
        dfind = np.mean(np.vstack([fonf[1:],fonf[:-1]]),0)
        #print(df)
        #print(dfind)

        if False:
            figure()
            hist(df,30)
            xlabel("duty cycle")
            ylabel("frequency")
            savefig("dutycyclehist.png")

        if self.metaData.has_key("dfbad"):
            db=self.metaData["dfbad"]
            print db
        else:
            db=[]

        self["dutyFactor"] = df
        self["dutyFactorInd"] = dfind
        self["strideDutyFactor"] = list()
        # Associate with cut strides:
        ind=0
        for idx0,idx1 in zip(self.strideIdx[self.firstStride:self.lastStride-1], \
            self.strideIdx[self.firstStride+1:self.lastStride]):
            fnd=np.where(np.logical_and(dfind>idx0,dfind<idx1))
            #print "Found:", fnd
            #print "Duty factors", df[fnd[0]]
            if len(fnd[0]) == 1:
                if ind not in db:
                    self["strideDutyFactor"].append(df[fnd[0]])
                    #print(df[fnd[0]])
                    if self.parent['trialType']=='Walk':
                        pylab.text(np.mean([idx0,idx1]),0,"_".join([str(df[fnd[0]]),str(ind)]))
                else:
                    print "Ignoring bad detection manually."
                    self["strideDutyFactor"].append([np.nan])
                    if self.parent['trialType']=='Walk':
                        pylab.text(np.mean([idx0,idx1]),0,"_".join(['MANUALBAD',str(ind)]))                    
            else:
                self["strideDutyFactor"].append([np.nan])
                if self.parent['trialType']=='Walk':
                    pylab.text(np.mean([idx0,idx1]),0,"_".join(['nan',str(len(fnd[0])),str(ind)]))
            ind=ind+1
        self["strideDutyFactor"] = np.array(self["strideDutyFactor"])

    def plotData(self):
        for limb in self.limbs:
            pylab.figure()
            pylab.title("Accelerometer Data for limb " + limb)
            for sig in self[limb+self.acc].transpose():
                pylab.plot(self[limb+self.acc].time,sig)
            pylab.legend(['X:alonglimb+=imudistal','Y:rostrocaudal:+=caudal','Z:lateralacrossbody:+=left'])
            pylab.grid()
        for limb in self.limbs:
            pylab.figure()
            pylab.title("Gyro Data for limb " + limb)
            for sig in self[limb+self.gyro].transpose():
                pylab.plot(self[limb+self.gyro].time,sig)                
            pylab.legend(['X:spinonlimbaxis','Y:legabductadduct','Z:limbrotswingstance'])
            pylab.grid()
        pylab.show()

    def plotZScoredData(self):
        pylab.figure()
        print "Plotting Z-scored accelerometer and gyro data"
        lines=list()
        limb = self.limbs[0]
        pylab.title("Z-Scored Signals (acc and gyro) for limb " + limb)
        dirs=["X","Y","Z"]
        for sig,dir in zip(self[limb+self.acc].transpose(),dirs):
            lines.extend(pylab.plot(self[limb+self.acc].time,(sig-nanmean(sig))/nanstd(sig),label=limb+"_acc_"+dir))
        for sig,dir in zip(self[limb+self.gyro].transpose(),dirs):
            lines.extend(pylab.plot(self[limb+self.gyro].time,(sig-nanmean(sig))/nanstd(sig),label=limb+"_gyro_"+dir))    
        #selUI(lines)
        #pylab.show()
    
    def plotDataSample(self):
        pylab.figure(figsize=(13,7.5))
        for j,limb in zip(range(len(self.limbs)+1),self.limbs+['GPS']):
            ax=pylab.subplot(1,5,j+1)
            wid = 2.4
            locs = np.array(range(6))*5
            labs = [limb+self.acc+axis for axis in ['X','Y','Z']] + [limb+self.gyro+axis for axis in ['X','Y','Z']]
            for k,sig in zip(range(3),self[limb+self.acc].transpose()):
                mid = np.mean([self[limb+self.acc].time[0],self[limb+self.acc].time[-1]])
                pylab.plot(self[limb+self.acc].time-mid,zscore(sig)+locs[k])
            for k,sig in zip(range(3,6),self[limb+self.gyro].transpose()):
                mid = np.mean([self[limb+self.gyro].time[0],self[limb+self.gyro].time[-1]])
                pylab.plot(self[limb+self.gyro].time-mid,zscore(sig)+locs[k])                
            ax.set_yticks(locs)
            ax.set_yticklabels(labs,rotation=90)
            ax.set_xlim([-wid/2,+wid/2])
            pylab.grid()
            if j==0:
                ax.set_ylabel('Signal (z-scored)')
            ax.set_xlabel('Time (sec)')
            ax.set_title(limb)
            ax.set_ylim((-5,30))
        ax.annotate( " ".join([self.parent.parent.iD,self.parent["trialType"], \
            self.parent["terrainType"],self.parent.iD]),xy=(0.5,1), xytext=(0,-10),\
            xycoords='figure fraction',textcoords='offset points',va='top',ha='center')
        pylab.savefig(osjoin(self.analysisDirectory, "_".join(["rawAccGyroSample", \
            self.parent.parent.iD,self.parent["trialType"],self.parent["terrainType"], \
            self.parent.iD,".png"])))
        #pylab.close()
    
    def plotPhaseComputation(self):
        pylab.figure(figsize=(13,7.5))
        for j,limb in enumerate(self.limbs):
            ax=pylab.subplot(1,4,j+1)
            wid = 5
            labs = ['raw gyro Z','raw acc Y','filt gyro Z','filt acc Y','phase']
            locs = np.array(range(len(labs)))*5
            mid = np.mean([self.masterTime[0],self.masterTime[-1]])
            for k,sig in zip(range(2),self.unfilteredSignals[limb]):
                pylab.plot(self.masterTime-mid,sig+locs[k])
            for k,sig in zip(range(2,4),self.filteredSignals[limb]):
                pylab.plot(self.masterTime-mid,sig+locs[k-2])
                pylab.plot(self.masterTime-mid,sig+locs[k])
            pylab.plot(self.masterTime-mid,self[limb+'Phase']+locs[4])
            ax.set_yticks(locs)
            ax.set_yticklabels(labs,rotation=90)
            ax.set_xlim([-wid/2,+wid/2])
            pylab.grid()
            if j==0:
                ax.set_ylabel('Signal')
            ax.set_xlabel('Time (sec)')
            ax.set_title(limb)
            ax.set_ylim((-5,5*(len(labs)+1)))
        ax.annotate( " ".join([self.parent.parent.iD,self.parent["trialType"], \
            self.parent["terrainType"],self.parent.iD]),xy=(0.5,1), xytext=(0,-10),\
            xycoords='figure fraction',textcoords='offset points',va='top',ha='center')
        pylab.savefig(osjoin(self.analysisDirectory, "_".join(["phaseComputation", \
            self.parent.parent.iD,self.parent["trialType"],self.parent["terrainType"], \
            self.parent.iD,".png"])))
        #pylab.close()
    
    def plotRelativePhases(self):
        sigs = list()
        # this is first limb to second from last zipped with second limb through last:
        #In [19]: zip(imud.limbs[:-1],imud.limbs[1:])
        #Out[19]: [('FL', 'BR'), ('BR', 'BL'), ('BL', 'FR')]
        labs = list()
        for l1,l2 in zip(self.limbs[:-1],self.limbs[1:]):
            sigs.append(self[l1+l2+"RelativePhase"])
            labs.append(l1+l2+"RelPh")
        rph = np.array(sigs).T
        pylab.figure(figsize=(13,7.5))
        ax1 = pylab.subplot2grid((4,1), (0, 0), rowspan=3)
        pylab.plot(self.masterTime-self.masterTime[0],rph)
        pylab.legend(labs)
        pylab.hlines([np.pi/2, np.pi,3*np.pi/2,2*np.pi],0,self.masterTime[-1]-self.masterTime[0])
        ax1.set_ylabel('Relative leg phase (rad)')
        ax1.set_xticks([])
        ax1.set_title(" ".join([self.parent.parent.iD,self.parent["trialType"], \
            self.parent["terrainType"],self.parent.iD]))
        ax1.set_ylim([0, np.pi*2])
        ax1.set_xlim((0,self.masterTime[-1]-self.masterTime[0]))
        
        # Velocity panel
        lpf = FiltFiltFilter(1/(self.masterTime[1]-self.masterTime[0]), 2, 3) # try a low pass as 4 hz... almost for gyro
        self["GPSSpeedInterp"] = lpf(wrappedInterpolate(self["GPSSpeed"].time,self["GPSSpeed"],self.masterTime,method="simple"))
        ax2 = pylab.subplot2grid((4,1), (3, 0), rowspan=1)
        pylab.plot(self.masterTime-self.masterTime[0],self["GPSSpeedInterp"])
        ax2.set_ylabel("Speed (m/s)")
        ax2.set_xlabel('Time (s)')
        ax2.set_xlim((0,self.masterTime[-1]-self.masterTime[0]))
        
        pylab.savefig(osjoin(self.analysisDirectory, "_".join(["relativePhases", \
            self.parent.parent.iD,self.parent["trialType"],self.parent["terrainType"], \
            self.parent.iD,".png"])))
            
    def _readData(self):
        self.acc="Acc"
        self.gyro="Gyro"
        self.limbs=["FL","BR","BL","FR"]
        
        '''
        Read data from a directory and store the variables
        '''
        self["GPSPos"] = list() # Store as a temporal signal eventually
        self["GPSError"] = list() # Store as a temporal signal array eventually
        self["GPSVel"] = list()
        self["GPSSpeed"] = list()
        self["GPSHorizSpeed"] = list()
        self["GPSHeading"] = list()
        self["GPSSpeedError"] = list()
        self["GPSHeadingError"] = list()
        self["GPSAcc"] = list() # Store as a temporal signal array eventually
        self["GPSGyro"] = list() # Store as a temporal signal array eventually

        self["GPSMag"] = list() # Store as a temporal signal array eventually, don't always have this
        
        self["FLAcc"] = list() # Store as a temporal signal array eventually
        self["FLGyro"] = list() # Store as a temporal signal array eventually
        self["FRAcc"] = list() # Store as a temporal signal array eventually
        self["FRGyro"] = list() # Store as a temporal signal array eventually
        self["BLAcc"] = list() # Store as a temporal signal array eventually
        self["BLGyro"] = list() # Store as a temporal signal array eventually
        self["BRAcc"] = list() # Store as a temporal signal array eventually
        self["BRGyro"] = list() # Store as a temporal signal array eventually
        
        
        # Read the GPS Velocity Data
        print "Reading GPS Velocity Data..."
        reader = csv.reader(open(osjoin(self.workingDirectory, self.fileName,"AGPS_UBXNAV-VELNED.txt"), "r"),delimiter="\t")
        ready = False
        ntime = list()
        for row in reader:
            if len(row)>0 and row[0]=="GPS Time (s)" and not ready:
                ready = True
            elif ready:
                ntime.append(float(row[0]))
                self["GPSVel"].append([float(x) for x in row[1:4]])
                self["GPSSpeed"].append(float(row[4]))
                self["GPSHorizSpeed"].append(float(row[5]))
                self["GPSHeading"].append(float(row[6]))
                self["GPSSpeedError"].append(float(row[7]))
                self["GPSHeadingError"].append(float(row[8]))
        self["GPSVel"]=TemporalSignal(self["GPSVel"],np.array(ntime),0)
        self["GPSSpeed"]=TemporalSignal(self["GPSSpeed"],np.array(ntime),0)
        self["GPSHorizSpeed"]=TemporalSignal(self["GPSHorizSpeed"],np.array(ntime),0)
        self["GPSHeading"]=TemporalSignal(self["GPSHeading"],np.array(ntime),0)
        self["GPSSpeedError"]=TemporalSignal(self["GPSSpeedError"],np.array(ntime),0)
        self["GPSHeadingError"]=TemporalSignal(self["GPSHeadingError"],np.array(ntime),0)
        
        # Read the GPS Position Data
        '''print "Reading GPS Position data..."
        reader = csv.reader(open(osjoin(self.workingDirectory, self.fileName,"AGPS_UBXNAV-POSLLH.txt"), "r"),delimiter="\t")
        ready = False
        ntime = list()
        for row in reader:
            if len(row)>0 and row[0]=="GPS Time (s)" and not ready:
                ready = True
            elif ready:
                ntime.append(float(row[0]))
                self["GPSPos"].append([float(x) for x in row[1:4]])
                self["GPSError"].append([float(x) for x in row[4:]])
        self["GPSPos"]=TemporalSignal(self["GPSPos"],ntime,0)
        self["GPSError"]=TemporalSignal(self["GPSError"],ntime,0)'''
        
        
        # Read the IMU Data
        print "Reading IMU Data (accelerometers and gyro)..."
        # GPS
        print "For GPS..."
        reader = csv.reader(open(osjoin(self.workingDirectory, self.fileName,"AGPS_ADC12_0_calib.txt"), "r"),delimiter="\t")
        ready = False
        ntime = list()
        for row in reader:
            if len(row)>0 and row[0]=="GPS Time (s)" and not ready:
                ready = True
            elif ready:
                ntime.append(float(row[0]))
                self["GPSAcc"].append([float(x) for x in row[1:4]])
                self["GPSGyro"].append([float(x) for x in row[4:]])
        self.masterTime=np.array(ntime)
        self["GPSAcc"]=TemporalSignal(self["GPSAcc"],np.array(ntime),0)
        self["GPSGyro"]=TemporalSignal(self["GPSGyro"],np.array(ntime),0) 
        
        # FR
        print "For Fore Right..."
        reader = csv.reader(open(osjoin(self.workingDirectory, self.fileName,"FR_ADC12_0_calib.txt"), "r"),delimiter="\t")
        ready = False
        ntime = list()
        for row in reader:
            if len(row)>0 and row[0]=="GPS Time (s)" and not ready:
                ready = True
            elif ready:
                ntime.append(float(row[0]))
                self["FRAcc"].append([float(x) for x in row[1:4]])
                self["FRGyro"].append([float(x) for x in row[4:]])
        self["FRAcc"]=TemporalSignal(self["FRAcc"],np.array(ntime),0)
        self["FRGyro"]=TemporalSignal(self["FRGyro"],np.array(ntime),0) 

        # FL
        print "For Fore Left..."
        reader = csv.reader(open(osjoin(self.workingDirectory, self.fileName,"FL_ADC12_0_calib.txt"), "r"),delimiter="\t")
        ready = False
        ntime = list()
        for row in reader:
            if len(row)>0 and row[0]=="GPS Time (s)" and not ready:
                ready = True
            elif ready:
                ntime.append(float(row[0]))
                self["FLAcc"].append([float(x) for x in row[1:4]])
                self["FLGyro"].append([float(x) for x in row[4:]])
        self["FLAcc"]=TemporalSignal(self["FLAcc"],np.array(ntime),0)
        self["FLGyro"]=TemporalSignal(self["FLGyro"],np.array(ntime),0) 

        # BR
        print "For Back Right..."
        reader = csv.reader(open(osjoin(self.workingDirectory, self.fileName,"BR_ADC12_0_calib.txt"), "r"),delimiter="\t")
        ready = False
        ntime = list()
        for row in reader:
            if len(row)>0 and row[0]=="GPS Time (s)" and not ready:
                ready = True
            elif ready:
                ntime.append(float(row[0]))
                self["BRAcc"].append([float(x) for x in row[1:4]])
                self["BRGyro"].append([float(x) for x in row[4:]])
        self["BRAcc"]=TemporalSignal(self["BRAcc"],np.array(ntime),0)
        self["BRGyro"]=TemporalSignal(self["BRGyro"],np.array(ntime),0) 
        
        # BL
        print "For Back Left..."
        reader = csv.reader(open(osjoin(self.workingDirectory, self.fileName,"BL_ADC12_0_calib.txt"), "r"),delimiter="\t")
        ready = False
        ntime = list()
        for row in reader:
            if len(row)>0 and row[0]=="GPS Time (s)" and not ready:
                ready = True
            elif ready:
                ntime.append(float(row[0]))
                self["BLAcc"].append([float(x) for x in row[1:4]])
                self["BLGyro"].append([float(x) for x in row[4:]])
        self["BLAcc"]=TemporalSignal(self["BLAcc"],np.array(ntime),0)
        self["BLGyro"]=TemporalSignal(self["BLGyro"],np.array(ntime),0) 
            
    def __init__(self, workingDirectory, analysisDirectory, metaData, parent=None):
        '''
        Constructor - Load stuff and compute phases
        '''
        BaseData.__init__(self, workingDirectory, analysisDirectory, metaData,parent)
        self.strideFrequencyEstimate = metaData["strideFrequencyEstimate"]
        if metaData.has_key("firstStride"):
            self.firstStride = metaData["firstStride"]
            if metaData.has_key("numStrides"):
                self.numStrides = metaData["numStrides"]
                self.lastStride = self.firstStride + self.numStrides
            else:
                self.lastStride = metaData["lastStride"]
        else:
            self.firstStride = 2
            self.lastStride = -2  
        self._readData()
        #self.plotZScoredData()
        #self.plotDataSample()
        self._getPhases()
        # add compute projections
        # add plot projections.
        #self.plotPhaseComputation()
        self._strideCut()
        self._estDutyFactor()
        #self.plotRelativePhases()
        #self.plotPhases(True)
        #self.plotPhases(False)
        #self.plot3DPhaseScatterPlot()
        #pylab.show()
        #self.singlePlotPhases(["FL","BR"],True)

def getSecondSampleIMUObject():
    '''This function returns an example of a motion capture dataModule
    object, based on the directory structure of Alexis' data from fish. 
    '''
    workingDir = osjoin("..","sampleData")    
    analysisDir = osjoin("..","sampleAnalysis","dataContainers")
    metaDataFile = osjoin("..",'sampleYAML','dataContainers','imuOtherSampleDataDog.yaml')
    stream = file(metaDataFile, 'r')
    metaData=yamlload(stream)
    stream.close()
    return(IMUData(workingDir,analysisDir,metaData['imuData']))         

def getSampleIMUObject():
    '''This function returns an example of a motion capture dataModule
    object, based on the directory structure of Alexis' data from fish. 
    '''
    workingDir = osjoin("..","sampleData")    
    analysisDir = osjoin("..","sampleAnalysis","dataContainers")
    metaDataFile = osjoin("..",'sampleYAML','dataContainers','imuSampleDataDog.yaml')
    stream = file(metaDataFile, 'r')
    metaData=yamlload(stream)
    stream.close()
    return(IMUData(workingDir,analysisDir,metaData['imuData']))  

if __name__=='__main__':
    '''
    Runs some tests using the sample data
    '''
    sampleIMUData = getSampleIMUObject()
