'''    
Nested Gaussian Process interpolation 
for oscillatory data with slow varying 
component.
Author: Simon Wilshin
Email: swilshin@rvc.ac.uk
Date: April 2013

TODO: Give this thing the markerset as a target?, so it can find limb + body markers. loop over appendages in marker set take all markers from set of appendages markers that contains this one. okey just use self.parent.resolve('')
TODO: Construct the phase required for each limb + body on first access, use mess from email
TODO: Make the actual interpolation of a time series work on gaps

TODO: Planning with Simon:
  - s=getNanRegions(timeseries) -> returns enumerated list of slices
  - p=getNeighboringPoints(listofslices) -> (rersultof getnanregions) return list of list of indices of closest points
  - ph=phaseEst(listofcoordinates) -> e.g. flatten list of all on limb, see below
    - list of def basePhaseEst(x)
      -returns phase
      - x is Nx(M*3) (numpy array) 
  - nGpMoCap -> just shuffling points and limbs, no interpolation
    - create nested gp int object inside interp, throw away when done (saving intepred marker)
    - have phase est from limb/body use that
    - inside interpolate:
      - loop over x,y,z
        - loop over nanregions
          - use time, phase, coordinate data (500 pts), to _init_ (non train mode)
            - use pred to get out the fill.
              - fill in diction with ngpint keyvalue.
  - nestGPinterpolator
'''

from sipylibs.phase import getPhaseEst
from sipylibs.nestedgpint import NestedGPInterpolator
from sipylibs.utils import getRegions, getNeighbouringPoints

# Will we need any of the below in this new version?
import scipy.cluster.hierarchy as hrchy
import numpy as np

from mDInterpret import loadMetaData
from motionCaptureData import MotionCaptureData
from utils import ensure_dir, fillNans
import pylab as pl
from os.path import join as osjoin
#import experimentTree

class nestedGPIntMoCapData(MotionCaptureData):
    '''
    Wraps the MotionCaptureData in methods that use a nested gaussian process
    to fill in blanks.
    
    Performs nested gaussian process 
    interpolation of data which can be 
    broken in to two components, a slow 
    varying components which is modelled
    using a gaussian process and a 
    faster varying oscillatory component 
    of known phase which is modelled using 
    a second gaussian process derived after 
    the first.
    This requires us to know the phase of 
    the oscillation and the time of both 
    a training data set and the set to be 
    interpolated.
    Makes use of the existing wrapper for 
    gaussian the Neumann GP library.
    '''
    def __init__(self, workingDirectory, analysisDirectory, metaData, parent):
    #def __init__(self,X0, Y0, M,sigmaSlow,sigmaFast,snoise,sfac=100.0,train=False):
        '''
        Requires the training data set for 
        the Guassian processes.
        X0 and Y0 are the original data from which the GP is built.
        M is ?
        sigmaSlow is the scale of the slow varying component of the signal (usually time, in sec).
          - For dog gait trans data, 1 s works well.
        sigmaFast is the scale of the fast varying component of the signal (usually phase in radians)
          - For dog gait trans data, 0.1 rad works well
        snoise is the noise scale.
          - For dog gait trans data, 0.1 works well
        sfac is the amount to supress the scale of the slow varying component in during the fast varying 
        interpolation.
        train determines if the fast varying hyperparameters are trained for before running.
    shape of X,Y,M; x=time?; what to use phase or pred?
    
    SHOULD DO THIS BY GAP WITHING THE TRIAL. DOH.
      - Find middle of the gap.
      - Find nearest 500 points. (use distnace ranked: so grabs 500 either side, or 500 to one side if it's an edge gap. Will it work if there's another gap? I suppose you fill it and move forward? Or not fill and move forward.
      
    INPUTS: 
            X0: NxD array; N=data points; D=dimensionality of input space. 1 col of X is time, other is phase
              - For dog: take all limb markers, vstack to Tx3N, subtract with spline X; Y, nan=zero, PCA on that. First Princomp should be the nice wiggly thing. Could diff that.
            Y0: Nx1 
            M: scalars, 1.0 
            sigmaSlow: 
            sigmaFast: 
            snoise: 
            sfac: 
            train: 
        '''
        MotionCaptureData.__init__(self, workingDirectory, analysisDirectory, metaData, parent)
        loadMetaData("M",self,self.metaData,1.0)
        loadMetaData("sigmaSlow",self,self.metaData,1.0)
        loadMetaData("sigmaFast",self,self.metaData,0.3)
        loadMetaData("snoise",self,self.metaData,0.1)
        loadMetaData("sfac",self,self.metaData,2)
        loadMetaData("ngptrain",self,self.metaData,False)
        
    def keys(self):
        return([k for k in self.data.keys() if k[-7:]!="_ngpint"])
        
    def has_key(self,k):
        return(k in self.keys())
      
    def interpolate(self,k,phaseEst):
        '''
        INPUTS:
                k = marker to interpolate
                phaseEst = estimate of phase to use in interpolation
        OUTPUT:
                stores filled marker data to self[k+'_ngpint']
                
          - s=getNanRegions(timeseries) -> returns enumerated list of slices
          - p=getNeighboringPoints(listofslices) -> (rersultof getnanregions) return list of list of indices of closest points
          - ph=phaseEst(listofcoordinates) -> e.g. flatten list of all on limb, see below
            - list of def basePhaseEst(x)
              -returns phase
              - x is Nx(M*3) (numpy array) 
          - nGpMoCap -> just shuffling points and limbs, no interpolation
            - create nested gp int object inside interp, throw away when done (saving intepred marker)
            - have phase est from limb/body use that
            - inside interpolate:
              - loop over x,y,z
                - loop over nanregions
                  - use time, phase, coordinate data (500 pts), to _init_ (non train mode)
                    - use pred to get out the fill.
                      - fill in diction with ngpint keyvalue.
          - nestGPinterpolator
        '''
        sigs = self.data[k]
        nSigs = np.copy(sigs)
        # Loop over coordinates: note data are 3xN so for loops over coord first
        for s,ns in zip(sigs,nSigs):        
            # Get regions we need to fill
            slicelist=getRegions(s)
            # Get points near each region to use to fill
            ptslist=getNeighbouringPoints(s,slicelist,N=200)
            # Loop over regions:
            for slicetofill,valpts in zip(slicelist,ptslist):
                # Make an interpolator
                # X0 is the time+phase for valid neighboring points
                ptsIdx = np.array(valpts[:,0],dtype=int)
                #print ptsIdx.shape
                #print s.time[ptsIdx].shape, phaseEst[ptsIdx].shape
                X0=np.vstack([s.time[ptsIdx],phaseEst[ptsIdx]]).T
                # Y0 is data for this valid region, or s[valptsidx]
                Y0 = valpts[:,1].reshape(valpts.shape[0],1)
                #print X0.shape, Y0.shape
                ngpi=NestedGPInterpolator(
                                          X0,
                                          Y0,
                                          self.M,
                                          self.sigmaSlow,
                                          self.sigmaFast,
                                          self.snoise,
                                          self.sfac
                                         )
                # FIXME: Should we "recursively" fill, e.g. fill next region on filled data?                                         
                # Fill the points in the new signals.                                       
                Xstar = np.vstack([s.time[slicetofill],phaseEst[slicetofill]]).T
                ns[slicetofill]=ngpi.pred(Xstar)[:,0]
        self.data[k+"_ngpint"] = nSigs
    
    def _buildBodyPhase(self):
        '''
        Get body markers, estimate phase from them, store
        TODO: Find the ms object once in _init_, store under self? Not sure if parent ready on init.
        '''
        ms=self.parent.resolve('*tMarkerSet')
        # Get Nx(M*3) array of marker data: N=samples, M=markers
        bmd=np.vstack([self.data[k] for k in ms['BodyMarkers']]).T
        self.data["ngpbodyphase"] = getPhaseEst(bmd)
        self.data["ngpbodyphase"] = (np.unwrap(self.data["ngpbodyphase"])/2.0)%(2.0*np.pi)

    def _buildLimbPhase(self,limb):
        '''
        Get limb markers, estimate phase from them, store
        limb is e.g. 'ForeLeft'
        '''
        ms=self.parent.resolve('*tMarkerSet')
        # Get Nx(M*3) array of marker data: N=samples, M=markers
        lmd=np.vstack([self.data[k] for k in ms[limb+'Markers']]).T
        self.data[limb+"_ngpphase"]=getPhaseEst(lmd,lpfac=1.0)
    
    def __getitem__(self,k):
        if not self.data.has_key(k+"_ngpint"):
            # See if user requested marker from body or appendage that we haven't trained for
            # Identify where marker comes from:
            ms=self.parent.resolve('*tMarkerSet')
            if k in ms["BodyMarkers"]:
                # It's body marker, build body phase for body marker NGP training if we don't have it.
                if not self.data.has_key("ngpbodyphase"):
                    if self.metaData["verbosity"]>0:
                        print "  nested GP building body phase for %s" % k
                    self._buildBodyPhase()
                # Whether we built phase now or earlier, now interpolate using it:
                if self.metaData["verbosity"]>0:
                    print "  nested GP interpolating %s" % k
                self.interpolate(k,self.data["ngpbodyphase"])
            else:
                # Not body marker. See if limb marker:
                limb=ms.getAppendageOfMarker(k)
                if limb is not None:
                    # Build limb phase for NGP training if we don't have it.
                    if not self.data.has_key(limb+"_ngpphase"):
                        if self.metaData["verbosity"]>0:
                            print "  nested GP building limb phase for %s" % k
                        self._buildLimbPhase(limb)
                    # Whether we built phase now or earlier, now interpolate using it:
                    if self.metaData["verbosity"]>0:
                        print "  nested GP interpolating %s" % k
                    self.interpolate(k,self.data[limb+"_ngpphase"])
                else:
                    raise KeyError("Key %s not part of body or appendages, cannot be used with ngpmocap at present"%k)
        return(self.data[k+"_ngpint"])
        
    def plotFillNGP(self,tofile=True,targetdir=None,suffix="nanfill",markers=None):
        if targetdir is None:
            targetdir=self.analysisDirectory
        if markers is None:
            ms=self.parent.resolve('*tMarkerSet')
            markers=ms.getAppendageAndBodyMarkers()
        else:
            if type(markers)==str:
                markers=[markers]
        for k in markers:
            sigs = self.data[k]
            if np.any( np.isnan(self.data[k]) ):
                if self.metaData["verbosity"]>0:
                    print "  Mocap object iD %s label %s has %d nans" % (self.iD,k,np.sum(np.isnan(self.data[k]))) 
                pl.figure()
                pl.plot(sigs.time,sigs.T)
                pl.plot(sigs.time,self[k].T)
                pl.plot(sigs.time[np.where(np.isnan(sigs[0]))[0]],self[k][:,np.where(np.isnan(sigs[0]))[0]].T,'o')
                pl.title(self.fileName[:-4])
                if tofile:
                    ensure_dir(targetdir)
                    targetfile = osjoin(targetdir, ".".join(["_".join([self.fileName[:-4],k,suffix]),"png"]))
                    print "  Writing image %s" % targetfile
                    pl.savefig(targetfile)
                    pl.close()
                else:
                    pl.show()

