'''
Created on 6 Feb 2011

@author: Simon Wilshin, swilshin@rvc.ac.uk

Data objects encapsulating information about a step perturbation an animal 
is running over. At present this class contains only one piece of information, 
the height of the step.
'''

from os.path import join as osjoin
from yaml import load as yamlload

from baseData import UnsampledBaseData

class StepPerturbationData(UnsampledBaseData):
    '''
    Contains information about a single compliant substrate assumed to be 
    omnipresent throughout a trial.
    '''
    
    def __init__(self, workingDirectory, analysisDirectory, metaData):
        UnsampledBaseData.__init__(self, workingDirectory, analysisDirectory, metaData)
        self.height=metaData["height"]
        
def getSampleStepPerturbationData():
    workingDir = osjoin("..","sampleData")    
    analysisDir = osjoin("..","sampleAnalysis","dataContainers")
    metaDataFile = osjoin("..",'sampleYAML','dataContainers','stepPerturbationSample.yaml')
    stream = file(metaDataFile, 'r')
    metaData=yamlload(stream)
    stream.close()
    return(StepPerturbationData(workingDir,analysisDir,metaData['stepPerturbation']))

if __name__=='__main__':
    '''
    Runs a test using the sample data (which is contained wholly in the YAML in this case)
    '''
    print "Processing sample step perturbation data"
    sampleStepPerturbationData=getSampleStepPerturbationData()
    print "Step Height: ", sampleStepPerturbationData.height, "mm."