'''
Created on 18 Jan 2011

@author: Simon Wilshin, swilshin@rvc.ac.uk

Data objects encapsulating information about the substrate an animal is on. 
At present there is only one class which contains only the compliance and 
type of a substrate. 
'''

from os.path import join as osjoin
from yaml import load as yamlload

from baseData import UnsampledBaseData

class SingleCompliantSubstrateData(UnsampledBaseData):
    '''
    Contains information about a single compliant substrate assumed to be 
    omnipresent throughout a trial.
    '''
    
    def __init__(self, workingDirectory, analysisDirectory, metaData):
        UnsampledBaseData.__init__(self, workingDirectory, analysisDirectory, metaData)
        self.name=metaData["surface"]
        self.k=metaData["surfacek"]
        
def getSampleSingleCompliantSubstrateData():
    workingDir = osjoin("..","sampleData")    
    analysisDir = osjoin("..","sampleAnalysis","dataContainers")
    metaDataFile = osjoin("..",'sampleYAML','dataContainers','compliantSubstrateSample.yaml')
    stream = file(metaDataFile, 'r')
    metaData=yamlload(stream)
    stream.close()
    return(SingleCompliantSubstrateData(workingDir,analysisDir,metaData['substrate']))

if __name__=='__main__':
    '''
    Runs a test using the sample data (which is contained wholly in the YAML in this case)
    '''
    print "Processing sample compliant substrate data"
    sampleSingleCompliantSubstrateData=getSampleSingleCompliantSubstrateData()
    print "Substrate Name: ", sampleSingleCompliantSubstrateData.name
    print "Substrate k: ", sampleSingleCompliantSubstrateData.k
