'''
Created on 26 Jan 2011

@author: Simon Wilshin swilshin@rvc.ac.uk

Interface for encapsulating weight data. This will allow us to use force plate 
objects as though they were weights as well as just raw numbers.
'''

from baseData import UnsampledBaseData
import conventions as conv

class WeightInterface(object):
    '''
    Specifies what methods and members a weight object should contain.
    Note this is an inferace, not a class. It amounts to a promise to provide 
    certain methods and members (those pertaining to a weight), and it should 
    not be instantiated in raw form.
    '''
    def __init__(self,weight):
        '''
        Initialise
        '''
        self.weight=weight
        if self.weight!=None:
            self.mass=weight/conv.g
        else:
            self.mass=None
        print "Subject weight is " + str(self.weight) + " and mass is " + str(self.mass) + "..."
        
class BasicWeightData(UnsampledBaseData,WeightInterface):
    '''
    Very basic container class for weights, is little more than just a number at the 
    moment.
    '''
    defaultIDExt = "wgt"
    def makePlots(self):
        print 'Making plots for BasicWeightData not implemented yet.'
    
    def __init__(self,workingDirectory, analysisDirectory, metaData,parent=None):
        UnsampledBaseData.__init__(self,workingDirectory, analysisDirectory, metaData,parent)
        WeightInterface.__init__(self,metaData["rawWeight"])
        
    def keys(self):
        return(["Weight","Mass"])
        
    def has_key(self,k):
        return(k in self.keys())
        
    def __getitem__(self,k):
        if k=="Weight":
            return(self.weight)
        elif k=="Mass":
            return(self.mass)
        else:
            raise(KeyError("No such key in weight data "+str(k)))
    
if __name__=='__main__':
    '''
    No testing because this is an interface and should never be directly instantiated.
    '''
    print "No testing because this is an interface and should never be directly instantiated."