'''
Created on 26 May 2011

@author: swilshin, energy calculations contributed by Josh Leech (2nd Year BVetSci RVC)
'''

import numpy as np
from scipy.stats import nanmean
from scipy.integrate import simps

from dy.logger import load as dylogload
from yaml import load as yamlload

from baseData import BaseData
from os.path import join as osjoin


def E(V,I):
    '''
    Energy formula as written by J. Leech
    '''
    return(V*I)

class XRLData(BaseData):
    '''
    Data pulled from a logger working on an xrl robot.
    '''
    def _readData(self):
        self["log"] = dylogload(osjoin(self.workingDirectory, self.fileName))
        self["motor"] = self["log"]["xrl1"]["motor"]
        self["motorI"] = self["motor"]["current"]
        self["motorV"] = self["motor"]["voltage"]
        self["leg"] = self["log"]["xrl1"]["leg"]
        self["legPos"] = self["leg"]["pos"]
        self["battery"] = self["log"]["xrl1"]["battery"]
        self["batteryI"] = self["battery"]["current"]
        self["batteryV"] = self["battery"]["voltage"]
    
    def _strideCut(self):
        # Find strides based on fore left limb phase
        self["meanStridePower"]=list()
        self["strideEnergy"]=list()
        strideIdx = [idx for e1,e2,idx in zip(self["legPos"].data[0][:-1],self["legPos"].data[0][1:],range(len(self["legPos"].data[0][:-1]))) if e1<0 and e2>0]
        print strideIdx, self["batteryI"].data[0], self["batteryV"].data[0]
        for idx0,idx1 in zip(strideIdx[:-1],strideIdx[1:]):
            self["meanStridePower"].append(
                                                   nanmean(E(self["batteryI"].data[0][idx0:idx1],self["batteryV"].data[0][idx0:idx1]))
                                          )
            self["strideEnergy"].append(
                                                   simps(E(self["batteryI"].data[0][idx0:idx1],self["batteryV"].data[0][idx0:idx1]))
                                          )            
        self["strideEnergy"] = np.array(self["strideEnergy"])
        self["meanStridePower"] = np.array(self["meanStridePower"])            
        print self["strideEnergy"],self["meanStridePower"]
    
    def __init__(self, workingDirectory, analysisDirectory, metaData):
        '''
        Constructor - Load stuff and compute phases
        '''
        BaseData.__init__(self, workingDirectory, analysisDirectory, metaData)
        self._readData()
        self._strideCut()

def getSampleXRLObject():
    '''
    This function returns an example of a motion capture dataModule
    object, based on the directory structure of Alexis' data from fish. 
    '''
    workingDir = osjoin("..","sampleData")    
    analysisDir = osjoin("..","sampleAnalysis","dataContainers")
    metaDataFile = osjoin("..",'sampleYAML','dataContainers','xrlSampleData.yaml')
    stream = file(metaDataFile, 'r')
    metaData=yamlload(stream)
    stream.close()
    return(XRLData(workingDir,analysisDir,metaData['xrlData']))  
