'''
Created on 18 Jan 2011

@author: Simon Wilshin, swilshin@rvc.ac.uk

A object which is defaultable will check it's parent for default settings 
and over write any with it's own configuration.
'''

from copy import deepcopy

class Defaultable(object):
    '''
    A object which is defaultable will check it's parent for default settings 
    and over write any with it's own configuration.
    ''' 
    def __init__(self, metaData):
        # Defaults apply to oneself, and to all children, and are overridden by
        # child defaults
        if self.parent==None:
            if metaData.has_key("defaults"):
                self.defaults = metaData["defaults"]
            else:
                self.defaults = dict()
        else:
            if metaData.has_key("defaults"):
                self.defaults = deepcopy(self.parent.defaults)
                self.defaults.update(metaData["defaults"])
            else:
                self.defaults = deepcopy(self.parent.defaults)
        self.metaData = deepcopy(self.defaults)
        
        # Child defaults only apply to children
        if metaData.has_key("childDefaults"):
            self.childDefaults = metaData["childDefaults"]
        else:
            self.childDefaults = dict()        
        if self.parent!=None:
            self.metaData.update(deepcopy(self.parent.childDefaults))
        self.metaData.update(metaData)
        self.metaData["defaults"] = self.defaults        

if __name__=='__main__':
    '''
    No tests here since the classes here are never supposed to be instantiated.
    '''
    print "No tests for Defaultable, this class should never be directly instantiated"