# This script is a modification of 'filtertrotspds.py' 
# It imports the csv data file created from 'roughGaits.py', orders it sensibly, 
# extracts peak density for 'walk', calcs the min and max speed we want to analyse, calcs 
# the stationary points, makes a plot and exports a csv with the data we want to keep & analyse in R.
# This is for the roughGaits manuscript, looking at walking strides.
from csv import reader
from csv import writer
from scipy.stats.kde import gaussian_kde
from numpy import *
from pylab import *

def filterspds(plt=False):
    # cd /mnt/hgfs/Michelle/RESEARCH/data/roughGaits_newanalysisforpaper/
    f=open("PROJroughGaitsMichelle.csv","r")
    r=reader(f)
    header = r.next()
    dat = [row for row in r]

    odat=array(dat)
    dat=dict(zip(header,odat.T))

    nonnumeric=["Dog","Terrain.Type","Gait.Type"]

    for k in dat:
        if not k in nonnumeric:
            dat[k]=[x if x != "NA" else "nan" for x in dat[k]]
            dat[k]=array([float(x) for x in dat[k]])

    dognames=list(set(dat["Dog"]))
    idxkeep=array([])

    for dog in dognames:
        v = dat["Speed"][logical_and(dog==dat["Dog"],"Walk"==dat["Gait.Type"])]

        vpdf = gaussian_kde(v)
        vr = linspace(min(v),max(v),10000)
        
        peakdens = (max(vpdf(vr)))
        modev = vr[argmax(vpdf(vr))]
        minspd = vr[vr<modev][argmin(abs(vpdf(vr[vr<modev])-peakdens/2))]
        maxspd = vr[vr>modev][argmin(abs(vpdf(vr[vr>modev])-peakdens/2))]
        dvpdf = diff(vpdf(vr))
        ddvpdf =diff(diff(vpdf(vr)))
        statpts = sign(dvpdf)[1:]!=sign(dvpdf)[:-1]

        for mv in vr[logical_and(ddvpdf>0,statpts)]:
            if mv < modev:
                if mv > minspd:
                    minspd = mv
            elif mv > modev:
                if mv < maxspd:
                    maxspd = mv
        # Here we made maxspd = the modev (peak of data) so we just get and export stuff that are slow trot i.e. below max peak. If in future want to analyse all the trots, can remove this line just below.
        #maxspd = modev
        idx = arange(dat["Speed"].size)
        idxadd = idx[logical_and(dog==dat["Dog"],"Walk"==dat["Gait.Type"])]
        idxkeep = hstack([idxadd[logical_and(v<maxspd,v>minspd)],idxkeep])
        if plt:
            figure() 
            plot(vr,vpdf(vr))
            vlines([modev,minspd,maxspd],0,max(vpdf(vr)))
        
    idxkeep=array(idxkeep,dtype=int)
    wf = open ("PROJroughGaitsWALKMichelleFILTERED.csv","wb")
    wr = writer(wf)
    wr.writerow(header)
    wr.writerows(odat[idxkeep])
    wf.close() 
    
if __name__ == "__main__":
    filterspds()

