'''
Created on 4 Feb 2011

@author: swilshin
'''

from scipy.integrate import cumtrapz
import numpy as np

from phase import deMod
from interpolation import wrappedInterpolate
from interpolation import nanInterpolation
from filters import FiltFiltFilter

def wrappedIntegrate(y,x):
    I=cumtrapz(y,x) # integrate
    ix=x[1:]
    return(wrappedInterpolate(ix, I, x,method='forcedSpline')) #interpolate

def detectDiscontinuities(y,delta):
    d=[]
    for i in range(len(y)-1):
        if abs(y[i]-y[i+1])>delta:
            d.append(i)
    return(d)

def numericallyDifferentiate(position,sampleRate,frequencyOfInterest=None,passFactor=4,filterOrder=5,phase=False,doFilter=True):
    if not phase:
        interpolatedPosition=nanInterpolation(position)
    else:
        interpolatedPosition=position%(2.0*np.pi)
    if doFilter and frequencyOfInterest!=None:
        bf=FiltFiltFilter(float(sampleRate),passFactor*float(frequencyOfInterest),filterOrder,'lowpass')
        if phase:
            interpolatedPosition=deMod(interpolatedPosition)
        if interpolatedPosition.ndim==1:
            filteredPosition=bf.filter(interpolatedPosition)
        else:
            filteredPosition=np.array([bf.filter(s) for s in interpolatedPosition])        
    else:
        filteredPosition=interpolatedPosition
    filteredPosition=np.ma.MaskedArray(filteredPosition,mask=np.isnan(position),fill_value=np.NaN)
    filteredPosition=filteredPosition.filled()
    if filteredPosition.ndim==1:
        v=(filteredPosition[2:]-filteredPosition[:-2])
        if phase:
            v = (v+np.pi)%(2.0*np.pi)-np.pi
        v = np.interp(range(0,len(position)),range(1,len(v)+1),v*sampleRate)
    else:
        v=np.array([(k[2:]-k[:-2]) for k in filteredPosition])
        if phase:
            v = (v+np.pi)%(2.0*np.pi)-np.pi
        v=sampleRate*np.array([np.interp(range(0,interpolatedPosition.shape[1]),range(1,v.shape[1]+1),k) for k in v])
    return(v/2.0) # Factor of two to correct for taking midpoint derivative

# Testing
eps=np.finfo(type(12.0)).eps

def testWrappedIntegrate():
    print "Testing the wrappedIntegrate function."
    t1 = wrappedIntegrate([1,1,1],[0,1,2])[-1]<2.0+2.0*eps and wrappedIntegrate([1,1,1],[0,1,2])[-1]>2.0-2.0*eps
    print "Test 0001 Passed: ", t1
    t2 = wrappedIntegrate([0,1,2],[0,1,2])[-1]<2.0+2.0*eps and wrappedIntegrate([0,1,2],[0,1,2])[-1]>2.0-2.0*eps
    print "Test 0002 Passed: ", t2
    t3 = wrappedIntegrate([0,2,3,4],[0,2,3,4])[-1]<8.0+8.0*eps and wrappedIntegrate([0,2,3,4],[0,2,3,4])[-1]>8.0-8.0*eps
    print "Test 0003 Passed: ", t3
    t4 = wrappedIntegrate([0,2,3,4,5,6,7,8],[0,2,3,4,5,6,7,8])[-1]<32.0+32.0*eps and wrappedIntegrate([0,2,3,4,5,6,7,8],[0,2,3,4,5,6,7,8])[-1]>32.0-32.0*eps
    print "Test 0004 Passed: ", t4    
    testsPassed=t1 and t2 and t3 and t3 and t4
    return(testsPassed)


def testDetectDiscontinuities():
    print "Testing the detectDiscontinuities function."
    d = detectDiscontinuities([0,1,2,3,0,1,2,3,0],2.5)
    t1 = len(d)==2
    print "Test 0001 Passed: ", t1
    try:
        t2 = d[0]==3
    except IndexError:
        t2 = False
    print "Test 0002 Passed: ", t2
    try:
        t3 = d[1]==7
    except IndexError:
        t3 = False
    print "Test 0003 Passed: ", t3    
    testsPassed=t1 and t2 and t3
    return(testsPassed)

def testNumericallyDifferentiate():
    print "Testing the numericallyDifferentiate function."
    f=numericallyDifferentiate(range(100),10,1,5)
    t1 = f[50]<20.01 and f[50]>19.99
    print "Test 0001 Passed: ", t1
    t2 = len(f)==100
    print "Test 0002 Passed: ", t2
    testsPassed = t1
    return(testsPassed)

def testCalculus():
    print "Testing calculus module, True indicates test passed."      
    t1=testWrappedIntegrate()
    t2=testDetectDiscontinuities()
    t3=testNumericallyDifferentiate()
    testsPassed=t1 and t2 and t3
    if testsPassed:
        print "All tests PASSED in module calculus."
    else:
        print "Some tests FAILED in module calculus!"
    return(testsPassed)

if __name__=='__main__':
    '''
    Run some tests, should print True if everything worked
    '''
    testCalculus()
