'''
Created on 12 Nov 2010

@author: Andrew Spence

Various unsorted utility functions for handling file input output.

Includes a function to read delimted files (either tsv or csv, for example)
with a single line header into a dictionary with keys equal to header entries
and values equal to a list of the data in that row.
'''
import csv
import sys

def dlmcol2dict(fn,delim=None):
    '''
     Reads a column oriented delimited data file that has a header into a dictionnary.
     Puts keys for each column header, with value containing a list with the data in
     each column.

     Example input file: 

     animnum,framenum,time,animspd,animlabspd,beltspeed,animangle,animx,animy,animvx,animvy
     0,195,12.49265,0.000,0.000,0.000,284.79,-0.1152,0.2196,0.0000,0.0000
     0,196,12.56016,0.000,0.000,0.000,285.55,-0.1152,0.2196,0.0000,0.0000
     0,197,12.62316,0.000,0.000,0.000,285.83,-0.1152,0.2196,0.0000,0.0000
     0,198,12.68519,0.000,0.000,0.000,286.13,-0.1152,0.2196,0.0000,0.0000
     0,199,12.75174,0.000,0.000,0.000,285.15,-0.1152,0.2196,0.0000,0.0000
     0,200,12.81806,0.000,0.000,0.000,285.18,-0.1152,0.2196,0.0000,0.0000

     Becomes a dictionary with keys animnum,time,etc., having values equal to lists containing
     that columns data.

     Useful improvements:
       Make it work with input arguments using name == main
       Otherwise a module you can import
       That is built on csv? that takes lots of input options about headers etc.
    '''
    f = open(fn, 'rt')
    try:
        if not delim:
            dialect = csv.Sniffer().sniff(f.read(1024))
            f.seek(0)
            reader = csv.reader(f,dialect)
        else:
            reader = csv.reader(f,delimiter=',')
        for row in reader:
            if reader.line_num == 1:
                fields=row
                dat=dict(zip(row,[list() for k in range(len(row))]))
                #print fields
                continue
            [dat[fields[k]].append(row[k]) for k in range(len(row))]
    finally:
        f.close()
    return dat
