'''
Created on 4 Feb 2011

@author: swilshin
'''

from scipy.signal import butter
from scipy.signal import lfilter
from scipy.signal import filtfilt
import numpy as np

class ButterFilter(object):
    ''' A wrapper for the scipy butterworth filter'''
    def __init__(self,sampleRate,db3,order=5,style="lowpass"):
        cutOff=(2.0*db3)/float(sampleRate)
        self.a,self.b=butter(order,cutOff,style)
        self.style=style
        
    def filter(self,x):
        if self.style=="lowpass" or self.style=="bandpass":
            return(lfilter(self.a,self.b,x-np.mean(x)) + np.mean(x))
        else:
            return(lfilter(self.a,self.b,x))
            
    def __call__(self,x):
        return(self.filter(x))

class FiltFiltFilter(object):
    ''' A wrapper for the scipy butterworth filter'''
    def __init__(self,sampleRate,db3,order=5,style="lowpass"):
        cutOff=(2.0*db3)/float(sampleRate)
        self.a,self.b=butter(order,cutOff,style)
        self.style=style
        
    def filter(self,x):
        if np.array(x).ndim==1:
            if self.style=="lowpass" or self.style=="bandpass":
                return(filtfilt(self.a,self.b,x-np.mean(x)) + np.mean(x))
            else:
                return(filtfilt(self.a,self.b,x))
        else:
            nx = []
            for xi in np.array(x):
                nx.append(self.filter(xi))
            return(np.array(nx))

            
    def __call__(self,x):
        return(self.filter(x))

def lpfiltbutter(data,samplerate,cutoff):
    '''A wrapper for zero-lag scipy butterworth filter without requiring
    instantiation of a class. For quick operation on numpy arrays.
    works down columns, e.g. first dimension of 2D numpy arrays.
    Defaults to 3rd order. Uses filtfilt.'''
    b,a=butter(3,float(cutoff)/float(samplerate),"lowpass")
    if data.ndim > 1:
        return(np.vstack([filtfilt(b,a,data[0:,k]) for k in range(data.shape[1])]).T)
    else:
        return(filtfilt(b,a,data))
        
# Testing
def testButterFilter():
    print "Testing the ButterFilter class."
    bf1=ButterFilter(10,5)
    sig=np.sin(2.0*np.pi*2*np.array(range(1000)))
    tsig=bf1.filter(sig)
    t1 = np.sum(sig-tsig)<1e-15 and np.sum(sig-tsig)>-1e-15
    print "Test 0001 Passed: ", t1
    bf2=ButterFilter(10,1)
    tsig=bf2.filter(sig) 
    t2 = np.sum(tsig)<1e-9 and np.sum(tsig)>-1e-9 
    print "Test 0002 Passed: ", t2
    testsPassed=t1 and t2
    return(testsPassed)

def testFilters():
    print "Testing filters module, True indicates test passed."      
    t1=testButterFilter()
    testsPassed=t1
    if testsPassed:
        print "All tests PASSED in module filters."
    else:
        print "Some tests FAILED in module filters!"
    return(testsPassed)
    
if __name__=='__main__':
    '''
    Run some tests, should print True if everything worked
    '''
    testFilters()