'''
Created on 27 Feb 2011

@author: swilshin

Finds the plane that fits a selection of markers and can tell you the distance 
of points to said plane. Should probably wrap into an object at some point.
'''

import numpy as np
from scipy.optimize import fmin

def makePlane(p):
    theta=p[1]
    psi=p[2]
    n=np.array([np.sin(psi)*np.sin(theta), np.cos(psi), np.sin(psi)*np.cos(theta)]) # Gimbal lock along Y axis!
    x0=p[0]*n
    return(n,x0)

def distanceFromPlane(p,x):
    n,x0=makePlane(p)
    return(np.dot(x,n)-np.dot(x0,n))

def planeChiSquared(p,x):
    n,x0=makePlane(p)
    return(np.sum([(np.dot(x,n)-np.dot(x0,n))**2]))

def findPlane(x):
    p0=[0,0,np.pi/2.0] # Straight down z-axis
    p=fmin(planeChiSquared,p0,args=tuple([x]))
    return p