'''
Created on 4 Feb 2011

@author: swilshin
'''

from scipy.interpolate import UnivariateSpline
from scipy.weave import inline
from scipy.weave import converters
import numpy as np

from randomIndexer import randomIndexer1D
from randomIndexer import randomIndexer2D
from splineInterpolationLimit import splineInterpolationLimit


def wrappedInterpolate(x,y,nx,method='spline',left=None,right=None):
    '''Wrapper for different interpolation types will fall back to simple linear interpolation if more
    sophisticated techniques fail'''
    nx=np.array(nx)
    x=np.array(x)
    y=np.array(y)
    s=None
    if method=='forcedSpline':
        method='spline'
        s=0;
    if y.ndim==1: 
        if len(y)<splineInterpolationLimit and len(nx)==len(y): # If y is excessively short intepolation is pointless 
            return y
        elif len(y)<splineInterpolationLimit:
            print "Using simple interpolation, signal is too short for spline interpolation"
            method='simple' # do not use spline interpolation on short sections it crashes
    else:
        if y.shape[1]<splineInterpolationLimit and nx.shape[0]==y.shape[1]: # If y is excessively short intepolation is pointless 
            return y
        elif y.shape[1]<splineInterpolationLimit:
            print "Using simple interpolation, signal is too short for spline interpolation"
            method='simple' # do not use spline interpolation on short sections it crashes        
    l=None
    r=None
    if not left==None:
        l=np.array(left)
        if y.ndim==2:
            if l.shape==():
                l=l.repeat(y.shape[0]-1)
    else:
        if y.ndim==2:
            l=[None]*y.shape[0]
    if not right==None:
        r=np.array(left)
        if y.ndim==2:
            if r.shape==():
                r=r.repeat(y.shape[0]-1)
    else:
        if y.ndim==2:
            r=[None]*y.shape[0]     
    if y.ndim==1:            
        if method=="simple":
            return(np.interp(nx,x,y,left=l,right=r))
        else:
            funcSpline=UnivariateSpline(x,y,s=s)
            # Spline interpolations
            return(funcSpline(nx))
    else:  
        zmd=zip(y,l,r)
        if method=="simple":
            return(np.array([np.interp(nx,x,[v for v in k if not np.isnan(v)],left=le,right=ri) for k,le,ri in zmd]))
        else:
            interpFunc=[]
            # Spline interpolation
            for k in y:
                funcSpline=UnivariateSpline(x,k,s=s)
                interpFunc.append(funcSpline(nx)) 
            return(np.array(interpFunc))


def nanInterpolation(d,method="spline",left=None,right=None):
    goodIndexSupport='''#include <math.h>'''
    goodIndexCode='''
    #define WINDOWS
    int i=0;
    int count=0;
    for(i=0;i<n;++i)
    {
        #ifdef WINDOWS
        if ( t_d(i) == t_d(i) )
        {
            goodIndex(count)=i;
            ++count;
        }
        #else
        if (isnan(t_d(i))!=1)
        {
            goodIndex(count)=i;
            ++count;
        }
        #endif
    }
    '''
    if np.array(d).ndim==1:
        t_d=np.array(d)
    else:
        t_d=np.array(sum(d,0))
    n=t_d.shape[0] #@UnusedVariable Used inline in weave
    goodIndex=np.zeros(np.isnan(t_d).tolist().count(False))
    inline(goodIndexCode,['t_d','n','goodIndex'], type_converters=converters.blitz,support_code=goodIndexSupport,libraries=['m'])
    if goodIndex.size==0:
        return(np.zeros_like(d))
    if np.array(d).ndim==1:
        goodFuncVals=randomIndexer1D(d,goodIndex)
        goodValues=goodFuncVals.shape[0]
        targetIndex=range(np.array(d).shape[0])
    else:
        goodFuncVals=randomIndexer2D(d,goodIndex)
        targetIndex=range(np.array(d).shape[1])
        goodValues=goodFuncVals.ndim-1
    '''print goodIndex,goodVals,randomIndexer1D(d,goodIndex)
    if d.ndim==1:
        goodIndex=[i for i in range(d.shape[0]) if not np.isnan(d[i])]
        goodFuncVals=np.array([k for k in d if not np.isnan(k)])
        targetIndex=range(d.shape[0])
        goodValues=goodFuncVals.shape[0]
    else:
        goodIndex=[i for i in range(d.shape[1]) if not np.isnan(sum(d,0))[i]]
        goodFuncVals=np.array([[j for j in k] for k in d.transpose() if not np.isnan(sum(k))]).transpose()
        targetIndex=range(d.shape[1])
        goodValues=goodFuncVals.ndim-1 # if all values are nan this collapses to zero'''
    if goodValues>0:
        return(wrappedInterpolate(goodIndex,goodFuncVals,targetIndex,method='forcedSpline'))
    else:
        print "Warning Could not complete nan interpolation, returning Zeros!"
        return(np.zeros_like(d))

def bulkInterpolation(d):
    ''' Interpolate the given list of lists so that each sublist is the same length assuming the end points 
        are the same
    '''
    maxLength=np.max([len(l) for l in d])
    nv=list()
    for v in d:
        originalIdx=np.array(range(len(v)))/float(len(v)-1.0)
        targetIdx=np.array(range(maxLength))/float(maxLength-1.0)
        nv.append(wrappedInterpolate(originalIdx,v,targetIdx))
    return(np.array(nv))
